#include "TranslationView.h"

#include <qcolor.h>
#include <qpainter.h>
#include <qxml.h>

#include "IO/DictHandler.h"
#include "Structure/CharData.h"
#include "Structure/Element.h"
#include "Structure/Tag.h"

TranslationView::TranslationView(QWidget* aParent, Element* aRoot)
	: QScrollView(aParent), Root(aRoot)
{
	setMinimumSize(700,300);
	resizeContents(0,0);
}

void TranslationView::drawContents(QPainter* p, int clipx, int clipy, int clipw, int cliph)
{
	p->setBackgroundColor(QColor(255,255,255));
	p->eraseRect(clipx,clipy,clipw,cliph);
	if (Root)
	{
		p->setPen(QColor(255,0,0));
		int LineHeight = p->fontMetrics().height();
		unsigned int StartLine = 0;
		if (clipy >= LineHeight)
			StartLine = clipy / LineHeight;
		unsigned int StopLine = (cliph/LineHeight)+StartLine+2;
		for (unsigned int i=StartLine; i<StopLine; ++i)
		{
			Element* E = Root->elementFromLine(i);
			if (E && E->isTranslationAccepted())
				p->setPen(QColor(0,127,0));
			else if (dynamic_cast<Tag*>(E))
				p->setPen(QColor(127,127,127));
			else
					p->setPen(QColor(0,0,0));
			if (E && (E == Selected))
			{
				p->fillRect(0,(i+1)*LineHeight-fontMetrics().ascent(),
						viewport()->width(),LineHeight,
						palette().active().highlight());
				p->setPen(QColor(255,255,255));
			}
			unsigned int x = 0;
			if (E)
				x = E->depth()*20;
			p->drawText(x,(i+1)*LineHeight,Root->contentFromLine(i,0));
			p->drawText(x+viewport()->width()/2,(i+1)*LineHeight,Root->contentFromLine(i,1));
		}
	}
}

Element* TranslationView::root()
{
	return Root;
}

void TranslationView::setRoot(Element* aRoot)
{
	Root = aRoot;
	Selected = 0;
	theDict.clear();
	if (Root)
	{
		resizeContents(width(),fontMetrics().height()*Root->height());
		Selected = Root->first();
		while (Selected && (dynamic_cast<CharData*>(Selected) == 0))
			Selected = Selected->next();
	}
	if (Selected)
	{
		emit charDataSelected(true);
		emit charDataChanged(dynamic_cast<CharData*>(Selected)->translation());
	}
	else
	{
		emit charDataSelected(false);
		emit charDataChanged("");
	}
}

void TranslationView::resizeEvent(QResizeEvent* anEvent)
{
	if (Root)
	{
		Root->updateHeightToWidth(fontMetrics(),viewport()->width()/2-10,0);
		resizeContents(viewport()->width()-10,fontMetrics().height()*Root->height());
		updateContents();
	}
}

void TranslationView::contentsMousePressEvent(QMouseEvent* anEvent)
{
	Selected = Root->elementFromLine(anEvent->y()/fontMetrics().height());
	updateContents();
	CharData* CD = dynamic_cast<CharData*>(Selected);
	if (CD)
		emit charDataChanged(CD->translation());
	else
		emit charDataChanged("");
	emit charDataSelected(CD != 0);
}

void TranslationView::setTranslationText(const QString& aText)
{
	CharData* CD = dynamic_cast<CharData*>(Selected);
	if (CD)
	{
		CD->setTranslation(aText);
		Root->updateHeightToWidth(fontMetrics(),viewport()->width()/2-10,0);
		resizeContents(viewport()->width()-10,fontMetrics().height()*Root->height());
		updateContents();
	}
}

void TranslationView::acceptTranslation()
{
	CharData* CD = dynamic_cast<CharData*>(Selected);
	if (CD)
	{
		CD->acceptTranslation(true);
		theDict.add(CD->original(), CD->translation());
		do
		{
			Selected = Selected->next();
		}
		while (Selected && (dynamic_cast<CharData*>(Selected) == 0));
		if (Selected)
			ensureVisible(0,Selected->start()*fontMetrics().height());
		else
			ensureVisible(0,Root->height()*fontMetrics().height());
		CD = dynamic_cast<CharData*>(Selected);
		if (CD)
			emit charDataChanged(CD->translation());
		else
			emit charDataChanged("");
		emit charDataSelected(CD != 0);
		updateContents();
	}
}

void TranslationView::translationIsOriginal()
{
	CharData* CD = dynamic_cast<CharData*>(Selected);
	if (CD)
	{
		CD->setTranslation(CD->original());
		Root->updateHeightToWidth(fontMetrics(),viewport()->width()/2-10,0);
		resizeContents(viewport()->width()-10,fontMetrics().height()*Root->height());
		acceptTranslation();
	}
}

void TranslationView::saveDictionary(const QString& aText)
{
	theDict.save(aText);
}

void TranslationView::loadDictionary(const QString& aText)
{
	DictHandler Parser(theDict);
	QFile xmlFile( aText );
	QXmlInputSource source( &xmlFile );
	QXmlSimpleReader reader;
	reader.setContentHandler( &Parser );
	reader.parse(source);
	if (Root)
	{
		Root->use(theDict);
		Root->updateHeightToWidth(fontMetrics(),viewport()->width()/2-10,0);
		resizeContents(viewport()->width()-10,fontMetrics().height()*Root->height());
		updateContents();
	}
}
